@@ -542,7 +542,6 @@ class CA( BiologicProgram ):
         params = {}
         for ch, ch_params in self.params.items():
             steps = len( ch_params[ 'voltages' ] )
-
             params[ ch ] = {
                 'Voltage_step':      ch_params[ 'voltages' ],
                 'vs_initial':        [ ch_params[ 'vs_initial' ] ]* steps,
@@ -566,26 +565,51 @@ class CA( BiologicProgram ):
     ):
         """
         Update voltage and duration parameters
+        
+        :param voltages: Dictionary of voltages list keyed by channel,
+            or single voltage to apply to all channels.
+        :param durations: Dictionary of durations list keyed by channel,
+            or single duration to apply to all channels.
+        :param vs_initial: Dictionary of vs. initials list keyed by channel,
+            or single vs. initial boolean to apply to all channels.
         """
-        steps = len( voltages )
+        # format params
+        if not isinstance( voltages, dict ):
+            # transform to dictionary if needed
+            voltages = { ch: voltages for ch in self.channels }
 
-        params = {
-            'Voltage_step': voltages,
-            'Step_number':  steps - 1
-        }
+        if ( durations is not None ) and ( not isinstance( voltages, dict ) ):
+            # transform to dictionary if needed
+            durations = { ch: durations for ch in self.channels }
 
-        if durations is not None:
-            params[ 'Duration_step' ] = durations
+        if ( vs_initial is not None ) and ( not isinstance( vs_initial, dict ) ):
+            # transform to dictionary if needed
+            vs_initial = { ch: vs_initial for ch in self.channels }
 
-        if vs_initial is not None:
-            params[ 'vs_initial' ] = vs_initial
+        # update voltages
+        for ch, ch_voltages in voltages.items():
+            if not isinstance( ch_voltages, list ):
+                # single voltage given, add to list
+                ch_voltages = [ ch_voltages ]
 
-        self.device.update_parameters(
-            self.channel,
-            'ca',
-            params,
-            types = self._parameter_types
-        )
+            steps = len( ch_voltages )
+            params = {
+                'Voltage_step': ch_voltages,
+                'Step_number':  steps - 1
+            }
+
+            if ( durations is not None ) and ( durations[ ch ] ):
+                params[ 'Duration_step' ] = durations[ ch ]
+
+            if ( vs_initial is not None ) and ( vs_initial[ ch ] ):
+                params[ 'vs_initial' ] = vs_initial[ ch ]
+
+            self.device.update_parameters(
+                ch,
+                'ca',
+                params,
+                types = self._parameter_types
+            )
 
 
 class CP( BiologicProgram ):
@@ -675,7 +699,6 @@ class CP( BiologicProgram ):
         params = {}
         for ch, ch_params in self.params.items():
             steps = len( ch_params[ 'currents' ] )
-
             params[ ch ] = {
                 'Current_step':      ch_params[ 'currents' ],
                 'vs_initial':        [ ch_params[ 'vs_initial' ] ]* steps,
@@ -699,39 +722,52 @@ class CP( BiologicProgram ):
     ):
         """
         Update current and duration parameters.
+
+        :param currents: Dictionary of currents list keyed by channel,
+            or single current to apply to all channels.
+        :param durations: Dictionary of durations list keyed by channel,
+            or single duration to apply to all channels.
+        :param vs_initial: Dictionary of vs. initials list keyed by channel,
+            or single vs. initial boolean to apply to all channels.
         """
-        steps = len( currents )
+        # format params
+        if not isinstance( currents, dict ):
+            # transform to dictionary if needed
+            currents = { ch: currents for ch in self.channels }
 
-        params = {
-            'Current_step': currents,
-            'Step_number':  steps - 1
-        }
+        if ( durations is not None ) and ( not isinstance( voltages, dict ) ):
+            # transform to dictionary if needed
+            durations = { ch: durations for ch in self.channels }
 
-        if durations is not None:
-            params[ 'Duration_step' ] = durations
+        if ( vs_initial is not None ) and ( not isinstance( vs_initial, dict ) ):
+            # transform to dictionary if needed
+            vs_initial = { ch: vs_initial for ch in self.channels }
 
-        if vs_initial is not None:
-            params[ 'vs_initial' ] = vs_initial
+        # update voltages
+        for ch, ch_currents in currents.items():
+            if not isinstance( ch_currents, list ):
+                # single voltage given, add to list
+                ch_currents = [ ch_currents ]
 
-        # set current ranges
-        if isinstance( params, dict ):
-            for ch, ch_params in params.items():
-                ch_params[ 'current_range' ] = self._get_current_range(
-                    ch_params[ 'currents' ]
-                )
+            steps = len( ch_currents )
+            params = {
+                'Current_step': ch_currents,
+                'Step_number':  steps - 1
+            }
 
-        else:
-            params[ 'current_range' ] = self._get_current_range(
-                params[ 'currents' ]
+            if ( durations is not None ) and ( durations[ ch ] ):
+                params[ 'Duration_step' ] = durations[ ch ]
+
+            if ( vs_initial is not None ) and ( vs_initial[ ch ] ):
+                params[ 'vs_initial' ] = vs_initial[ ch ]
+
+            self.device.update_parameters(
+                ch,
+                'cp',
+                params,
+                types = self._parameter_types
             )
 
-        self.device.update_parameters(
-            self.channel,
-            'cp',
-            params,
-            types = self._parameter_types
-        )
-
 
     def _get_current_range( self, currents ):
         """
@@ -740,39 +776,39 @@ class CP( BiologicProgram ):
         :param currents: List of currents.
         :returns: ec_lib.IRange corresponding to largest current.
         """
-        i_max = max( currents )
+        i_max = max( abs( currents ) )
 
         if i_max < 100e-12:
             i_range = ecl.IRange.p100
 
-        elif max_i < 1e-9:
+        elif i_max < 1e-9:
             i_range = ecl.IRange.n1
 
-        elif max_i < 10e-9:
+        elif i_max < 10e-9:
             i_range = ecl.IRange.n10
 
-        elif max_i < 100e-9:
+        elif i_max < 100e-9:
             i_range = ecl.IRange.n100
 
-        elif max_i < 1e-6:
+        elif i_max < 1e-6:
             i_range = ecl.IRange.u1
 
-        elif max_i < 10e-6:
+        elif i_max < 10e-6:
             i_range = ecl.IRange.u10
 
-        elif max_i < 100e-6:
+        elif i_max < 100e-6:
             i_range = ecl.IRange.u100
 
-        elif max_i < 1e-3:
+        elif i_max < 1e-3:
             i_range = ecl.IRange.m1
 
-        elif max_i < 10e-3:
+        elif i_max < 10e-3:
             i_range = ecl.IRange.m10
 
-        elif max_i < 100e-3:
+        elif i_max < 100e-3:
             i_range = ecl.IRange.m100
 
-        elif max_i <= 1:
+        elif i_max <= 1:
             i_range = ecl.IRange.a1
 
         else:
@@ -861,13 +897,12 @@ class CALimit( BiologicProgram ):
 
     def run( self, retrieve_data = True ):
         """
-        :param retrieve_data: Automatically retrieve and disconenct form device.
+        :param retrieve_data: Automatically retrieve and disconnect from device.
             [Default: True]
         """
         params = {}
         for ch, ch_params in self.params.items():
             steps = len( ch_params[ 'voltages' ] )
-
             params[ ch ] = {
                 'Voltage_step':      ch_params[ 'voltages' ],
                 'vs_initial':        [ ch_params[ 'vs_initial' ] ]* steps,
@@ -900,11 +935,11 @@ class CALimit( BiologicProgram ):
         """
         Update voltage and duration parameters.
 
-        :param voltages: Dictionary of voltages keyed by channel,
+        :param voltages: Dictionary of voltages list keyed by channel,
             or single voltage to apply to all channels.
-        :param durations: Dictionary of durations keyed by channel,
+        :param durations: Dictionary of durations list keyed by channel,
             or single duration to apply to all channels.
-        :param vs_initial: Dictionary of vs. initials keyed by channel,
+        :param vs_initial: Dictionary of vs. initials list keyed by channel,
             or single vs. initial boolean to apply to all channels.
         """
         # format params
@@ -923,11 +958,10 @@ class CALimit( BiologicProgram ):
         # update voltages
         for ch, ch_voltages in voltages.items():
             if not isinstance( ch_voltages, list ):
-                # single voltage given, add to list
+                # single voltage given, make list
                 ch_voltages = [ ch_voltages ]
 
             steps = len( ch_voltages )
-
             params = {
                 'Voltage_step': ch_voltages,
                 'Step_number':  steps - 1
@@ -1154,7 +1188,7 @@ class JV_Scan( BiologicProgram ):
         defaults = {
             'start': 0,
             'step':  0.01,
-            'rate':  10,
+            'rate':  0.01,
             'average': False
         }
         channels = kwargs[ 'channels' ] if ( 'channels' in kwargs ) else None
