@@ -20,12 +20,18 @@ from .lib import ec_lib  as ecl
 
 # # Biologic Device
 
+HardwareConfiguration = namedtuple( 'HardwareConfiguration', [
+    'connection',  # electrode connection
+    'mode'  # channel mode
+] )
+
 
 TechParams = namedtuple( 'TechParams', [
     'technique',
     'parameters'
 ] )
 
+
 TechData = namedtuple( 'TechData', [
     'data',
     'info',
@@ -116,7 +122,6 @@ class BiologicDevice:
         :returns: List of ChannelInfo objects.
         """
         if not self.is_connected():
-            # device not connected
             raise EcError( -1 )
 
         return [
@@ -124,6 +129,26 @@ class BiologicDevice:
             for ch, available in enumerate( self.plugged )
         ]
 
+
+    @property
+    def hardware_configuration( self ):
+        """
+        :returns: List of HardwareConf objects.
+        """
+        if not self.is_connected():
+            raise EcError( -1 )
+
+        if self.kind is not ecl.DeviceCodes.KBIO_DEV_SP300:
+            raise RuntimeError( 'Hardware configuration is only available for SP-300 devices.' )
+
+        confs = {
+            ch: self.channel_configuration( ch ) if available else None
+            for ch, available in enumerate( self.plugged )
+        }
+
+        return confs
+
+
     @property
     def techniques( self ):
         """
@@ -161,7 +186,7 @@ class BiologicDevice:
             )
 
         except EcError as err:
-            if err.value is -9:
+            if err.value == -9:
                 # ECLab firmware loaded
                 pass
 
@@ -216,6 +241,41 @@ class BiologicDevice:
         return connected
 
 
+    def channel_configuration( self, ch ):
+        """
+        :param ch: Channel to configure.
+        :returns: HardwareConf object for the channel or None if channel is not available.
+        """
+        if not self.is_connected():
+            raise EcError( -1 )
+
+        if not self.plugged[ ch ]:
+            return None
+
+        conf = ecl.get_hardware_configuration( self.idn, ch )
+        return HardwareConfiguration(
+            connection = ecl.ElectrodeConnection( conf.Conn ),
+            mode = ecl.ChannelMode( conf.Ground )
+        )
+
+
+    def set_channel_configuration( self, ch, mode, connection ):
+        """
+        Sets the hardware configuration for the given channel.
+
+        :param ch: Channel to set.
+        :param mode: ChannelMode.
+        :param connection: ElectrodeConnection.
+        """
+        if not self.is_connected():
+            raise EcError( -1 )
+
+        if self.kind is not ecl.DeviceCodes.KBIO_DEV_SP300:
+            raise RuntimeError( 'Hardware configuration is only available for SP-300 devices.' )
+
+        ecl.set_hardware_configuration( self.idn, ch, mode, connection )
+
+
     def load_technique(
         self,
         ch,
@@ -241,7 +301,7 @@ class BiologicDevice:
             # not connected
             raise RuntimeError( 'Device not connected.')
 
-        first = ( index is 0 )
+        first = ( index == 0 )
 
         if types is not None:
             params = ecl.cast_parameters( params, types )
@@ -407,7 +467,7 @@ class BiologicDevice:
         """
         self.__idn        = None # device identifier
         self.__plugged    = None # list of plugged in channels
-        self.__techniques = None #list of channel techniques
+        self.__techniques = None # list of channel techniques
 
 
 
@@ -503,6 +563,41 @@ class BiologicDeviceAsync:
             for ch, available in enumerate( self.plugged )
         ]
 
+
+    @property
+    async def hardware_configuration( self ):
+        """
+        :returns: List of HardwareConf objects.
+        """
+        if not self.is_connected():
+            raise EcError( -1 )
+
+        if self.kind is not ecl.DeviceCodes.KBIO_DEV_SP300:
+            raise RuntimeError( 'Hardware configuration is only available for SP-300 devices.' )
+
+        # collect channel configurations for available channels
+        available_chs = []
+        ch_confs = []
+        for ch, available in enumerate( self.plugged ):
+            if available:
+                available_chs.appned( ch )
+                ch_confs.append( self.channel_configuration( ch ) )
+
+        ch_confs = await asyncio.gather( *ch_confs )
+
+        # collect results for all channels
+        confs = {
+            ch: (
+                ch_confs[ available_chs.index( ch ) ] 
+                if ch in available_chs else 
+                None
+            )
+            for ch, available in enumerate( self.plugged )
+        }
+
+        return confs
+
+
     @property
     def techniques( self ):
         """
@@ -540,7 +635,7 @@ class BiologicDeviceAsync:
             )
 
         except EcError as err:
-            if err.value is -9:
+            if err.value == -9:
                 # ECLab firmware loaded
                 pass
 
@@ -585,6 +680,41 @@ class BiologicDeviceAsync:
         return await ecl.is_connected_async( self.idn )
 
 
+    async def channel_configuration( self, ch ):
+        """
+        :param ch: Channel to configure.
+        :returns: HardwareConf object for the channel or None if channel is not available.
+        """
+        if not self.is_connected():
+            raise EcError( -1 )
+
+        if not self.plugged[ ch ]:
+            return None
+
+        conf = await ecl.get_hardware_configuration_async( self.idn, ch )
+        return HardwareConfiguration(
+            connection = ecl.ElectrodeConnection( conf.Conn ),
+            mode = ecl.ChannelMode( conf.Ground )
+        )
+
+
+    async def set_channel_configuration( self, ch, mode, connection ):
+        """
+        Sets the hardware configuration for the given channel.
+
+        :param ch: Channel to set.
+        :param mode: ChannelMode.
+        :param connection: ElectrodeConnection.
+        """
+        if not self.is_connected():
+            raise EcError( -1 )
+
+        if self.kind is not ecl.DeviceCodes.KBIO_DEV_SP300:
+            raise RuntimeError( 'Hardware configuration is only available for SP-300 devices.' )
+
+        await ecl.set_hardware_configuration_async( self.idn, ch, mode, connection )
+
+
     async def load_technique(
         self,
         ch,
@@ -610,7 +740,7 @@ class BiologicDeviceAsync:
             # not connected
             raise RuntimeError( 'Device not connected.')
 
-        first = ( index is 0 )
+        first = ( index == 0 )
 
         if types is not None:
             params = await ecl.cast_parameters_async( params, types )
@@ -776,4 +906,4 @@ class BiologicDeviceAsync:
         """
         self.__idn        = None # device identifier
         self.__plugged    = None # list of plugged in channels
-        self.__techniques = None #list of channel techniques
+        self.__techniques = None # list of channel techniques
